/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/

using System;
using System.IO;

using LLCryptoLib;
using LLCryptoLib.Utils;
using LLCryptoLib.Crypto;

namespace LLCryptoLibExamples
{
	/// <summary>
	/// Encrypt a file with asynchronous feedback on operations
	/// </summary>
	sealed class FeedbackExample
	{
		public static void EncryptFileWFeedBack()
		{
			// 0. Set delegate
			CallbackPoint cbp = new CallbackPoint(FeedbackExample.UpdateCounter);

			// 1. Create random file
			FileInfo rndFile = FeedbackExample.CreateRandomFile();
			Console.WriteLine("File created.");

			// 2. Encrypt with Rijndael
			StreamARC4512 cryptAlgorithm = LLCryptoLib.Crypto.StreamAlgorithmFactory.ArcFour512;
			LLCryptoLib.Crypto.StreamCrypter crypter = new StreamCrypter(cryptAlgorithm);
			crypter.GenerateKeys("password");
			string encryptedFile = rndFile.FullName + ".enc";
			crypter.EncryptDecrypt(rndFile.FullName, encryptedFile, true, cbp);
			Console.WriteLine("");
			Console.WriteLine("File encrypted into " + encryptedFile);
		}

		private static void UpdateCounter(int i, string message)
		{
			Console.Write("#"); // operation feedback

			if (message!=null)
			{
				Console.WriteLine("");
				Console.WriteLine(message);
			}
		}

		private static FileInfo CreateRandomFile()
		{
            FileManager fm = FileManager.Reference;
			FileInfo newFile = new FileInfo(@".\xxxlocal");
			fm.SaveFile(newFile.FullName, LLCryptoLib.Utils.RandomString.Get(200000),false);
			return newFile;
		}
	}
}
