/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/

using System;
using System.Text;
using System.IO;

using LLCryptoLib;
using LLCryptoLib.Utils;
using LLCryptoLib.Hash;

namespace LLCryptoLibExamples
{
	/// <summary>
    /// Compute Hash code
	/// </summary>
	public static class HashExample
	{

        /// <summary>
        /// Print the hash codes of a string with different encodings
        /// </summary>
        public static void HashString()
        {
            const string abc = "ABCDEFGHIJKL";

            Console.WriteLine("Hash of string '"+abc+"'");

            using (Hash hashObj = new Hash())
            {
                hashObj.SetAlgorithm(AvailableHash.MD5);
                Console.WriteLine("< UTF-8 Encoding >");
                Console.WriteLine("MD5: {0}", hashObj.ComputeHashStyle(abc, HexEnum.CLASSIC));
                hashObj.SetAlgorithm(AvailableHash.SHA1);
                Console.WriteLine("SHA1: {0}", hashObj.ComputeHashStyle(abc, HexEnum.CLASSIC));

                Console.WriteLine("< Unicode Encoding >");
                hashObj.SetAlgorithm(AvailableHash.MD5);
                Console.WriteLine("MD5: {0}", hashObj.ComputeHashStyle(abc, HexEnum.CLASSIC));
                hashObj.SetAlgorithm(AvailableHash.SHA1);
                Console.WriteLine("SHA1: {0}", hashObj.ComputeHashStyle(abc, HexEnum.CLASSIC));
            }
        }

        /// <summary>
        /// Create some hash codes
        /// </summary>
		public static void Hash()
		{
			// 1. Create a random file
            FileInfo fileToHash = RandomFile.CreateRandomBinary();
            if (fileToHash == null)
            {
                throw new Exception("Can't create test file");
            }

			Console.WriteLine("Created random file: {0}", fileToHash.FullName);

			// 2. Print the MD5 hash code
			IHash hashObject = new Hash();
			hashObject.SetAlgorithm(AvailableHash.MD5);
            Console.WriteLine("MD5 hash: {0}", hashObject.ComputeHashFileStyle(fileToHash.FullName, HexEnum.UNIX));

			// 3. Print the SHA1 hash code
			hashObject.SetAlgorithm(AvailableHash.SHA1);
            Console.WriteLine("SHA1 hash: {0}", hashObject.ComputeHashFileStyle(fileToHash.FullName, HexEnum.UNIX));

			// 4. Print the TIGER hash code
			hashObject.SetAlgorithm(AvailableHash.TIGER);
            Console.WriteLine("TIGER hash: {0}", hashObject.ComputeHashFileStyle(fileToHash.FullName, HexEnum.UNIX));

			// 5. Delete file
			fileToHash.Delete();
            Console.WriteLine("Deleted random file: {0}", fileToHash.FullName);
			
		}

	}
}
