/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/


using System;
using System.Collections.Generic;
using System.Text;
using LLCryptoLib.Security.Certificates;
using LLCryptoLib.Crypto;
using LLCryptoLib.Utils;
using System.IO;
using System.Security.Cryptography;
using LLCryptoLib.Hash;

namespace LLCryptoLibExamples
{
    class PublicKeyEncryption
    {

        const string DIGITAL_CERTIFICATE_PEM =
            //"-----BEGIN CERTIFICATE-----" +
            @"MIICLTCCAZagAwIBAgIQueAz2dvGyaVKXHpef1oeADANBgkqhkiG9w0BAQQFADAYMRYwFAYDV" +
            @"QQDEw1MaXR0bGVMaXRlIENBMB4XDTA3MDcyNjIyMDAwMFoXDTA5MDgyODIyMDAwMFowFTETMB" +
            @"EGA1UEAxMKSm9obiBTbWl0aDCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEAone4Ff+2YJu" +
            @"CioqzO0M0XfLtVCAKfM4jPROq0OxwdizAkcitawtnavEuhJckjOYjySRhRct4h4zrRl6Of8WA" +
            @"Fka0cNqn3jxZeWMNwzCUYtY+XCaZftvHEPJnoNGodoTF365ribbv3u380A8lbC6N9ugp1+ocX" +
            @"DmxO8aRmz889ckCAwEAAaN7MHkwDQYDVR0KBAYwBAMCBkAwHQYDVR0lBBYwFAYIKwYBBQUHAw" +
            @"MGCCsGAQUFBwMEMEkGA1UdAQRCMECAEIHvwn6I2FYrGQ0w8c2RZRWhGjAYMRYwFAYDVQQDEw1" +
            @"MaXR0bGVMaXRlIENBghDn+A1cCXxbnkP2HVDZmuXnMA0GCSqGSIb3DQEBBAUAA4GBAGU+rjJX" +
            @"yogOMoVnAWDZJT2Gm4dzBYYd+jpoOM9VuUss7n8z3dlfHocGVstWmwQ9M/elCu9H2gqxbFlwJ" +
            @"EyI4JkTcZL2YuNS5LvT88/l1R02vp+OQkfhJTqaVWFzeyo603T51RYxksl9S0vg4nngT2Z4uY" +
            @"m/KHNHLZGtPmSV8U1r";
            //"-----END CERTIFICATE-----"

        const string XML_KEY_PAIR =
            @"<RSAKeyValue>" +
                @"<Modulus>one4Ff+2YJuCioqzO0M0XfLtVCAKfM4jPROq0OxwdizAkcitawtnavEuhJckjOYjy" +
                    @"SRhRct4h4zrRl6Of8WAFka0cNqn3jxZeWMNwzCUYtY+XCaZftvHEPJnoNGodoTF365ribb" +
                    @"v3u380A8lbC6N9ugp1+ocXDmxO8aRmz889ck=" +
                "</Modulus>" +
                "<Exponent>" +
                    @"AQAB" +
                "</Exponent>" +
                @"<P>4pcEvuP8Zwo+GJxN2B13ANaoyys0Efe/iIjRLE7iBwX6dVFIIr7n3eezxD3Tipist8kw6ahleguMdiL8MmOeaw==</P>" +
                @"<Q>t44WgvqKDfq6ngPkCMYFGAYUj3t6ekD3EqAqOtk9BCAtkxtMV4GWj+YXh7l5mbRq0UiZ4MR6QbxYIizx2BDhmw==</Q>" +
                @"<DP>J7OkTjouMHO3+6Hvdc1723bBLpAMWJWbRrYKSIQt0GM4xjOOux/OCrCda+YVWbR0luM2ocoytZaubsikfcO8rw==</DP>" +
                @"<DQ>JNMguQ5CfCN9JQ5qxUfU5cjyfwaqv7Qg3V+8mSnUD+vUuNqVKkkZR61FszkGqEuF0o7aMDm6vU1n8RNLQanBWQ==</DQ>" +
                @"<InverseQ>1mGDW/uyb7oTRbBQWKMhV4VV70xxanL70ukMef/CenpjcnD3pCXlXN3aczXJUZiYwdVUuNEYwB+wzS/JaiF23g==</InverseQ>" +
                @"<D>LoQwhLvuL8LWoWwM3Q7k03FWWhvCZPFwTXBwG9lWab9uc08Nt/RP6+KZ+wR7Rowt+s0bvKwBGm5SfpdcHWEixsiRUWSp" +
                @"YSSFIUI9LLmwYKAgbyLdpp7glyUbnqjbjjAYdrBkSzrQy8RnselgLyr0YAAOFtPXnsE+KePokfrm2jE=" + 
                @"</D>" +
          "</RSAKeyValue>";

        /// <summary>
        /// This example shows how to encrypt a file using the public key of the
        /// receiver, and then how to decrypt it with the private key of the
        /// receiver.
        /// </summary>
        public static void PublicKeyExample()
        {
            string originalFileHash = PublicKeyEncryption.EncryptFile();
            Console.WriteLine("Original file MD5: " + originalFileHash);

            string decryptedFileHash = PublicKeyEncryption.DecryptFile();
            Console.WriteLine("Decrypted file MD5: " + decryptedFileHash); // hashes must be equal!

            PublicKeyEncryption.DeleteAllTestFiles();
        }

        /// <summary>
        /// This example show how to digitally sign a file using the private key
        /// of the sender. Then the public key of the sender is used to verify that
        /// the file has been tampered.
        /// The file is then intentionally tampered, to show that the verification
        /// fails in such case.
        /// </summary>
        public static void DigitalSignExample()
        {
            // Generates a file with signature appended
            string signedFile = PublicKeyEncryption.SignFile();

            // Verify signature
            if (PublicKeyEncryption.VerifySignedFile(signedFile))
            {
                Console.WriteLine("File verification is successful. ");
            }
            else
            {
                Console.WriteLine("File verification FAILED. ");
            }

            // Now tamper the signed file
            Console.WriteLine("Now tampering file... ");
            string tamperedFile = PublicKeyEncryption.TamperFile(signedFile);

            // Re-verify: should fail
            if (PublicKeyEncryption.VerifySignedFile(tamperedFile))
            {
                Console.WriteLine("File verification is successful. ");
            }
            else
            {
                Console.WriteLine("File verification FAILED. ");
            }


            DeleteAllTestFiles();
        }

        private static string TamperFile(string filePath)
        {
            string tamperedFileName = filePath+".tampered";

            byte[] bin = new byte[4096];
            int len = 1; 
            long rdlen = 0;

            using (FileStream fsInput = new FileStream(filePath, FileMode.Open))
            {
                long totlen = fsInput.Length;

                using (FileStream fsOutput = new FileStream(tamperedFileName, FileMode.Create))
                {

                    while (rdlen < totlen)
                    {
                        len = fsInput.Read(bin, 0, 4096);
                        bin[200] = 0; // tamper here!
                        fsOutput.Write(bin, 0, len);
                        rdlen += len;
                    }
	
                }
            }

            return tamperedFileName;
        }

        private static string EncryptFile()
        {
            // 1. Create an AsymmetricCrypter object
            AsymmetricCrypter ac = new AsymmetricCrypter(SupportedStreamAlgorithms.BLOWFISH);

            // 2. Load public key of receiver
            LLCertificate cert = LLCertificate.CreateFromPemString(DIGITAL_CERTIFICATE_PEM);
            RSA publicKey = cert.PublicKey;

            // 3. Create and save random file
            FileManager fm = FileManager.Reference;
            fm.SaveFile("test.file", RandomString.Get(200));

            // 4. Encrypt it
            ac.Encrypt(new FileInfo("test.file"), new FileInfo("test.file.encrypted"), publicKey);

            // 5. Return the MD5 of the original file to confront it later
            Hash hs = new Hash();
            hs.SetAlgorithm(AvailableHash.MD5);
            return hs.ComputeHashFile("test.file");
        }


        private static string DecryptFile()
        {
            // 1. Create an AsymmetricCrypter object
            AsymmetricCrypter ac = new AsymmetricCrypter(SupportedStreamAlgorithms.BLOWFISH);

            // 2. Load private key of receiver
            RSACryptoServiceProvider encryptor = new RSACryptoServiceProvider();
            RSA privateKey = AsymmetricCrypter.KeyFromXml(encryptor, XML_KEY_PAIR);

            // 3. Load encrypted file
            ac.Decrypt(new FileInfo("test.file.encrypted"), new FileInfo("test.file.decrypted"), privateKey);

            // 4. Return the MD5 of the decrypted file to confront it later
            Hash hs = new Hash();
            hs.SetAlgorithm(AvailableHash.MD5);
            return hs.ComputeHashFile("test.file.decrypted");

        }

        private static string SignFile()
        {

            const string ORIGINAL_FILE = "test.file";
            const string FILE_SIGNED = "test.file.signed";

            // 1. Load private key of signer
            RSACryptoServiceProvider encryptor = new RSACryptoServiceProvider();
            RSA privateKey = AsymmetricCrypter.KeyFromXml(encryptor, XML_KEY_PAIR);

            // 2. Create and save random file
            FileManager fm = FileManager.Reference;
            fm.SaveFile(ORIGINAL_FILE, RandomString.Get(1000));
            Console.WriteLine("Created file: test.file (" + FileManager.FileSize(ORIGINAL_FILE) + " bytes)");

            // 3. Sign file
            AsymmetricCrypter.Sign(new FileInfo(ORIGINAL_FILE),
                                   new FileInfo(FILE_SIGNED),
                                   privateKey, AvailableHash.MD5);
            Console.WriteLine("Signed file: test.file.signed (" + FileManager.FileSize(FILE_SIGNED) + " bytes)"); 

            return FILE_SIGNED;
        }

        private static bool VerifySignedFile(string signedFilePath)
        {
            // 1. Load public key of signer
            LLCertificate cert = LLCertificate.CreateFromPemString(DIGITAL_CERTIFICATE_PEM);
            RSA publicKey = cert.PublicKey;

            // 2. Verify signed file
            return AsymmetricCrypter.VerifySignature(new FileInfo(signedFilePath), // file to be verified
                                                     publicKey,                    // public key of signer
                                                     AvailableHash.MD5,            // hash function
                                                     false);                       // do not remove signature from file
        }

        private static void DeleteAllTestFiles()
        {
            DeleteIfExist("test.file");
            DeleteIfExist("test.file.encrypted");
            DeleteIfExist("test.file.decrypted");
            DeleteIfExist("test.file.signed");
        }

        private static void DeleteIfExist(string fileName)
        {
            if (File.Exists(fileName))
            {
                File.Delete(fileName);
            }
        }

    }
}
