﻿/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/

using System;
using System.IO;
using LLCryptoLib.Shred;


namespace LLCryptoLibExamples
{
    class ShredExample
    {

        public static void ShredFile()
        {
            ShredExample sex = new ShredExample();

            // We choose to use HMG method
            IShredMethod method = ShredMethods.Get(AvailableShred.HMGIS5ENH);

            sex.Shred(method);
        }

        private void Shred(IShredMethod method)
        {
             Shredder sh = new Shredder(new Random());
             FileInfo fi = this.CreateRandomFile();
             if (fi.Exists)
             {
                 // Wipe file
                 Console.WriteLine("Shredding file "+ fi.FullName);
                 sh.WipeFile(fi, method, false);

                 // Verify that the written byte sequence is the same of the shredding method
                 byte lastByte = method.GetSequence()[method.Passes - 1];
                 Console.WriteLine(("Verifying " + sh.LastShreddedPath + " (" + method.ToString() + ")"));
                 if (this.VerifyShreddedFile(sh.LastShreddedPath, lastByte))
                 {
                    Console.WriteLine("ERROR: Shredded file contains byte different than last byte of the method");
                 }
                 else
                 {
                     Console.WriteLine("File has been wiped on this with method "+method.Name);
                 }

                 // Remove entry from the FAT/NTFS table
                 fi.Delete();
             }
             else
             {
                 Console.WriteLine("ERROR: cannot create " + fi.FullName);
             }
         }

        private bool VerifyShreddedFile(string filepath, byte lastByte)
        {
            byte[] theFile;
            using (FileStream fs = new FileStream(filepath, FileMode.Open, FileAccess.Read))
            {
                using (BinaryReader br = new BinaryReader(fs))
                {
                    theFile = br.ReadBytes((int)fs.Length);
                }
            }

            bool errorFound = false;

            foreach (byte b in theFile)
            {
                if (b != lastByte)
                {
                    errorFound = true;
                }
            }

            return errorFound;
        }

        private FileInfo CreateRandomFile()
        {
            Console.WriteLine("Creating random file.");
            return RandomFile.CreateRandomBinary();
        }

    }

}
