/*
    LLCryptoLib Examples
    Advanced .NET Encryption and Hashing Library
    
	 
    Copyright (C) 2003-2008 LittleLite Software
    
*/

using System;
using System.Collections;
using System.Text;

using LLCryptoLib;
using LLCryptoLib.Utils;
using LLCryptoLib.Crypto;

namespace LLCryptoLibExamples
{
    static class TextEncryptionExample
    {
        /// <summary>
        /// Encrypts a string.
        /// </summary>
        public static void EncryptString()
        {
            string origString = "This is a LLCryptoLib test";
            Console.WriteLine("Original string: "+origString);
            Console.WriteLine();

            string encrypted;
            string decrypted;

            // TextROT13 TEXT TRANSFORMATION
            Console.WriteLine("ROT13 TRANSFORM");
            TextAlgorithmParameters parms = new TextAlgorithmParameters(3);
            TextCrypter textEncrypter = TextCrypterFactory.Create(SupportedTextAlgorithms.ROT13,parms);        
            encrypted = textEncrypter.TextEncryptDecrypt(origString, true);
            Console.WriteLine("Encrypted string: " + encrypted);
            decrypted = textEncrypter.TextEncryptDecrypt(encrypted, false);
            Console.WriteLine("Decrypted string: " + decrypted);
			Console.WriteLine();

            // PLAYFAIR TRANSFORMATION
            Console.WriteLine("PLAYFAIR TRANSFORM");
            TextAlgorithmParameters parms_zero = new TextAlgorithmParameters(3);
            TextCrypter textEncrypter_zero = TextCrypterFactory.Create(SupportedTextAlgorithms.PLAYFAIR, parms_zero);
            encrypted = textEncrypter_zero.TextEncryptDecrypt(origString, true);
            Console.WriteLine("Encrypted string: " + encrypted);
            decrypted = textEncrypter_zero.TextEncryptDecrypt(encrypted, false);
            Console.WriteLine("Decrypted string: " + decrypted);
            Console.WriteLine();

            // RIJNDAEL TEXT TRANSFORMATION (Hexadecimal output)
            Console.WriteLine("RIJNDAEL TRANSFORM");
            TextAlgorithmParameters parms_two = new TextAlgorithmParameters("llcryptopassword");
            TextCrypter textEncrypter_two = TextCrypterFactory.Create(SupportedTextAlgorithms.RIJNDAEL, parms_two);
            encrypted = textEncrypter_two.HexEncryptDecrypt(origString, true, HexEnum.CLASSIC);
            Console.WriteLine("Encrypted string: " + encrypted);
            decrypted = textEncrypter_two.HexEncryptDecrypt(encrypted, false, HexEnum.CLASSIC);
            Console.WriteLine("Decrypted string: " + decrypted);
			Console.WriteLine();

            // 3DES TEXT TRANSFORMATION (Base64 output)
            Console.WriteLine("3DES TRANSFORM");
            TextAlgorithmParameters parms_three = new TextAlgorithmParameters("llcryptopassword");
            TextCrypter textEncrypter_three = TextCrypterFactory.Create(SupportedTextAlgorithms.TRIPLEDES, parms_three);
            encrypted = textEncrypter_three.Base64EncryptDecrypt(origString, true);
            Console.WriteLine("Encrypted string: " + encrypted);
            decrypted = textEncrypter_three.Base64EncryptDecrypt(encrypted, false);
            Console.WriteLine("Decrypted string: " + decrypted);
			Console.WriteLine();
        }
    }
}
